#include <lpc214x.h>
#include "usb/usbdesc.h"
#include "usb/usbapi.h"

#ifdef HID_MODE
extern unsigned char abReportDesc[];
extern int abReportDescSize;

static BOOL HIDHandleStdReq(TSetupPacket *pSetup, int *piLen, unsigned char **ppbData)
{
	unsigned char bType, bIndex;

	if ((pSetup->bmRequestType == 0x81) && (pSetup->bRequest == REQ_GET_DESCRIPTOR))
	{	
		bType  = GET_DESC_TYPE(pSetup->wValue);
		bIndex = GET_DESC_INDEX(pSetup->wValue);
		switch (bType) 
		{
		case DESC_HID_REPORT:
			// report
			*ppbData = abReportDesc;
			*piLen = abReportDescSize;
			break;
		case DESC_HID_HID:
		case DESC_HID_PHYSICAL:
		default:
			return USBGetDescriptor(pSetup->wValue, pSetup->wIndex, piLen, ppbData);
		}
		return TRUE;
	}
	return FALSE;
}
#endif

volatile BYTE USBNewIn = 0;
volatile BYTE USBNewOut = 0;

static void _HandleBulk(unsigned char bEP, unsigned char bEPStatus)
{
	if (bEP == BULK_IN_EP)
		USBNewIn = 1;
	else if (bEP == BULK_OUT_EP)
		USBNewOut = 1;
}

int USBRead(unsigned char *pbBuf)
{
	while (USBNewOut == 0);
	USBNewOut = 0; 
	return USBHwEPRead(BULK_OUT_EP, pbBuf, MAX_PACKET_SIZE_OUT);
}

int USBWrite(unsigned char *pbBuf)
{
	USBHwEPWrite(BULK_IN_EP, pbBuf, MAX_PACKET_SIZE_IN);
	while (USBNewIn == 0); 
	USBNewIn = 0;
	return 0;
}

int main(void)
{
	IODIR0	|= (unsigned long)1 << 30;
	IOCLR0   = (unsigned long)1 << 30;

	USBInit();
	USBRegisterDescriptors(abDescriptors);
#ifdef HID_MODE
	USBRegisterCustomReqHandler(HIDHandleStdReq);
#endif
	USBHwRegisterEPIntHandler(BULK_OUT_EP,  _HandleBulk);
	USBHwRegisterEPIntHandler(BULK_IN_EP,   _HandleBulk);
	USBHwConnect(TRUE);
	while(1);
	return 0;
}
