#include "usbapi.h"
#include "usbdesc.h"

#ifdef HID_MODE
const U8 abReportDesc[] = 
{
	0x06, 0xA0, 0xFF,			// Usage Page (vendor-defined)
	0x09, 0x00,					// Usage (vendor-defined) // Undefined
	0xA1, 0x01,					// Collection (Application)
		0x15, 0x00,				// Logical Minimum (0)
		0x26, 0xff, 0x00,		// Logical Maximum (255)
		// Input
		0x09, 0x03,				// Usage (vendor-defined)
			0x95, MAX_PACKET_SIZE_IN,	// Report Count (RS)
			0x75, 0x08,			// Report Size (8 bits)
			0x81, 0x02,			// Input (Data, Variable, Absolute)
		// Output
		0x09, 0x04,				// Usage (vendor-defined)
			0x95, MAX_PACKET_SIZE_OUT,	// Report Count (RS)
			0x75, 0x08,			// Report Size (8 bits)
			0x91, 0x02,			// Output (Data, Variable, Absolute)
	0xC0,						// End Collection
};

const int abReportDescSize = sizeof(abReportDesc);
#endif 

const U8 abDescriptors[] = {
// 1) Device descriptor 
	0x12,						// Size
	DESC_DEVICE,
	LE_WORD(0x0200),			// bcdUSB
#ifdef HID_MODE
	0x00,						// bDeviceClass
#else
	0xFF,						// bDeviceClass
#endif
	0x00,						// bDeviceSubClass
	0x00,						// bDeviceProtocol
	MAX_PACKET_SIZE0,			// bMaxPacketSize
#ifdef HID_MODE
	LE_WORD(0xFFF0),			// idVendor
	LE_WORD(0x0001),			// idProduct
#else
	LE_WORD(0xFFF0),			// idVendor
	LE_WORD(0x0002),			// idProduct
#endif
	LE_WORD(0x0100),			// bcdDevice
	0x01,						// iManufacturer
	0x02,						// iProduct
	0x03,						// iSerialNumber
	0x01,						// bNumConfigurations

// 2) Configuration descriptor
	0x09,						// Size
	DESC_CONFIGURATION,
#ifdef HID_MODE
	//LE_WORD(0x29),				// wTotalLength
	LE_WORD(0x200),			// wTotalLength (!!! bug !!!)
#else
	LE_WORD(0x20),				// wTotalLength
#endif
	0x01,						// bNumInterfaces
	0x01,						// bConfigurationValue
	0x00,						// iConfiguration
	0x80,						// bmAttributes
	0x32,						// bMaxPower

// 3) Interface descriptor
	0x09,						// Size
	DESC_INTERFACE, 
	0x00,		 				// bInterfaceNumber
	0x00,						// bAlternateSetting
	0x02,						// bNumEndPoints
#ifdef HID_MODE
	0x03,   					// bInterfaceClass = HID
#else
	0xFF,						// bInterfaceClass
#endif
	0x00,						// bInterfaceSubClass
	0x00,						// bInterfaceProtocol
	0x00,						// iInterface

#if defined( HID_MODE )
// 4) HID descriptor
	0x09, 
	DESC_HID_HID, 				// bDescriptorType = HID
	LE_WORD(0x0110),			// bcdHID
	0x00,   					// bCountryCode
	0x01,   					// bNumDescriptors = report
	DESC_HID_REPORT,   			// bDescriptorType
	LE_WORD(sizeof(abReportDesc)),
#endif

#if defined( HID_MODE )
// 4) EndPoint Descriptor (int in)
	0x07,
	DESC_ENDPOINT,				// EP descriptor
	BULK_IN_EP,					// bEndpointAddress
	0x03,   					// bmAttributes = INT
	LE_WORD(MAX_PACKET_SIZE_IN),// wMaxPacketSize
	1,							// bInterval = 1 ms 
// 5) EndPoint Descriptor (int out)
	0x07,   		
	DESC_ENDPOINT,   			// EP descriptor
	BULK_OUT_EP,				// bEndpointAddress
	0x03,   					// bmAttributes = INT
	LE_WORD(MAX_PACKET_SIZE_OUT),// wMaxPacketSize
	1,							// bInterval = 1 ms 
#else
// 4) EndPoint Descriptor (bulk out)
	0x07,						// Size
	DESC_ENDPOINT,
	BULK_OUT_EP,				// bEndpointAddress
	0x02,						// bmAttributes = BULK
	LE_WORD(MAX_PACKET_SIZE_OUT),// wMaxPacketSize
	0,							// bInterval

// 5) EndPoint Descriptor (bulk in)
	0x07,						// Size
	DESC_ENDPOINT,
	BULK_IN_EP,					// bEndpointAddress
	0x02,						// bmAttributes = BULK
	LE_WORD(MAX_PACKET_SIZE_IN),// wMaxPacketSize
	0,							// bInterval
#endif

// 5) String descriptors
	0x04,						// Size
	DESC_STRING,
	LE_WORD(0x0409),			// wLocale

// 7) Manuf string
	11*2+2,						// Size
	DESC_STRING,
	'S', 0, // 2
	'a', 0, // 4
	'm', 0, // 6
	'p', 0, // 8
	'l', 0, // 10
	'e', 0, // 12
	' ', 0, // 14
	'I', 0, // 16
	'n', 0, // 18
	'c', 0, // 20
	'.', 0, // 22

// 8) Product string
	12*2+2,						// Size
	DESC_STRING,
	'T', 0, // 2
	'e', 0, // 4
	's', 0, // 6
	't', 0, // 8
	' ', 0, // 10
	'L', 0, // 12
	'P', 0, // 14
	'C', 0, // 16
	'2', 0, // 18
	'1', 0, // 20
	'4', 0, // 22
	'x', 0, // 24

	// Terminator
	0
};

